/*
    This file is part of the KContacts framework.
    Copyright (c) 2018 Volker Krause <vkrause@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef KCONTACTS_ISOTOCOUNTRYMAP_P_H
#define KCONTACTS_ISOTOCOUNTRYMAP_P_H

#include <QString>

namespace KContacts {

/* ISO code to country name string table lookup table entry. */
struct IsoToCountryIndex
{
    explicit constexpr inline IsoToCountryIndex(const char isoCode[2], int offset)
        : m_c1(isoCode[0])
        , m_c2(isoCode[1])
        , m_offset(offset)
    {
    }

    char m_c1;
    char m_c2;
    uint16_t m_offset;
};

}

#endif
